==> cisp-1.0.1.tar.gz: cisp-1.0.1/cisp.c <==
/**
 * @file cisp.c
 * Compact In System Programmer
 * @author Marko Mkel (marko.makela@nic.funet.fi)
 */

/* Copyright  2002 Marko Mkel.

   This file is part of CISP, a utility that allows in-system programming
   of electronic components.

   CISP is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   CISP is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   The GNU General Public License is often shipped with GNU software, and
   is generally kept in a file called COPYING or LICENSE.  If you do not
   have a copy of the license, write to the Free Software Foundation,
   59 Temple Place, Suite 330, Boston, MA 02111 USA. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <errno.h>

#include "comm.h"
#include "commsel.h"
#include "spi.h"
#include "ihex.h"
#include "srec.h"

/** Physical data source */
typedef int (*psrc_t) (const struct comm*, const struct avr*,
		       unsigned, unsigned char*);
/** Physical data destination */
typedef int (*pdest_t) (const struct comm*, const struct avr*,
			unsigned, unsigned char);
/** Stream data destination */
typedef int (*sdest_t) (FILE*, const struct comm*, const struct avr*,
			unsigned, psrc_t);

/** Signal handler
 * @param num	the signal number
 */
static void
sig (int num)
{
  terminate ();
  signal (num, SIG_DFL);
  raise (num);
}

/** the communication primitives */
static const struct comm* comm = 0;
/** the detected device */
static const struct avr* avr = 0;

/** Save a memory area
 * @param len	the length of the memory area
 * @param name	the file name ('-'=stdout)
 * @param src	the data source
 * @param dest	the data destination
 * @return	zero on success, nonzero otherwise
 */
static int
save (unsigned len,
      char* name,
      psrc_t src,
      sdest_t dest)
{
  int st;
  FILE* f;
  f = *name == '-' && !name[1] ? stdout : fopen (name, "w");
  if (!f) {
    fputs (name, stderr), perror (": fopen(writing)");
    return -1;
  }

  st = (*dest) (f, comm, avr, len, src);
  if (f != stdout)
    fclose (f);
  return st;
}

/** Load a memory area
 * @param name	the file name ('-'=stdin)
 * @param dest	the data destination
 * @param done	function to invoke at the end of data
 * @return	zero on success, nonzero otherwise
 */
static int
load (char* name,
      pdest_t dest,
      int (*done) (const struct comm*, const struct avr*))
{
  int c;
  FILE* f = *name == '-' && !name[1] ? stdin : fopen (name, "r");
  if (!f) {
    fputs (name, stderr), perror (": fopen(reading)");
    return -1;
  }
  c = getc (f);
  ungetc (c, f);
  switch (c) {
  default:
    fputs (name, stderr), fputs (": unknown format\n", stderr);
    c = EOF;
    break;
  case EOF:
    c = 0;
    break;
  case ':':
    c = ihex_input (f, comm, avr, dest, done);
    break;
  case 'S':
    c = srec_input (f, comm, avr, dest, done);
    break;
  }
  if (f != stdin)
    fclose (f);
  return c;
}

/** The main function
 * @param argc	argument count
 * @param argv	argument vector
 * @return	0 if successful
 */
int
main (int argc, char** argv)
{
  char** param;
  atexit (terminate);
  signal (SIGINT, sig);
  signal (SIGTERM, sig);

#ifndef NO_SIGNAL
  signal (SIGINT, sig);
  signal (SIGTERM, sig);
#endif /* NO_SIGNAL */

  /* process the command-line parameters */
  for (param = argv; ++param < &argv[argc]; ) {
    if (**param != '-' || !(*param)[1] || (*param)[2])
      goto Unrecognized;

    switch ((*param)[1]) {
    case 'c':
      if (param + 2 >= &argv[argc]) {
      Missing:
	fprintf (stderr, "%s: argument missing\n", *param);
	goto Usage;
      }
      if (comm)
	terminate ();
      if (!(comm = establish (param[1], param[2])))
	return 2;
      param += 2;
    reset:
      if (enable (comm)) {
	fputs ("cisp: failed to enable the programming mode\n", stderr);
	return 3;
      }
      if ((avr = detect (comm)))
	fputs (avr->name, stderr), fputs (" detected\n", stderr);
      break;
    case 'C':
      if (param + 1 >= &argv[argc])
	goto Missing;
      else if (!comm) {
      Comm:
	fprintf (stderr, "%s: no -c parameter given\n", *param);
	goto Usage;
      }
      else if (strlen (param[1]) != 8) {
      nothex:
	fprintf (stderr, "-C: expecting 8 hexadecimal characters, not %s\n",
		 param[1]);
	goto Usage;
      }
      else {
	unsigned char cmd[4], recv[4];
	unsigned i;
	for (i = 0; i < 8; i++) {
	  unsigned char digit = param[1][i];
	  if (digit >= '0' && digit <= '9')
	    digit -= '0';
	  else if (digit >= 'A' && digit <= 'F')
	    digit -= 'A' - 10;
	  else if (digit >= 'a' && digit <= 'f')
	    digit -= 'a' - 10;
	  else
	    goto nothex;
	  if (i & 1)
	    cmd[i >> 1] |= digit;
	  else
	    cmd[i >> 1] = digit << 4;
	}
	sendrecv (comm, cmd, recv, "-C");
	fprintf (stderr, "-C %02X%02X%02X%02X: got %02X%02X%02X%02X\n",
		 cmd[0], cmd[1], cmd[2], cmd[3],
		 recv[0], recv[1], recv[2], recv[3]);
      }
      param++;
      break;
    case 'e':
      if (!comm)
	goto Comm;
      if (erase (comm)) {
	fputs ("cisp: failed to erase the device\n", stderr);
	return 3;
      }
      goto reset;
      break;
    case 's': case 'i':
    case 'S': case 'I':
      if (param + 1 >= &argv[argc])
	goto Missing;
      if (!comm)
	goto Comm;
      if (!avr) {
      Detect:
	fprintf (stderr, "%s: no device has been detected\n", *param);
	goto Usage;
      }
      if (save ((*param)[1] == 's' || (*param)[1] == 'i'
		? avr->flash_size
		: avr->eeprom_size,
		param[1],
		(*param)[1] == 's' || (*param)[1] == 'i'
		? read_flash
		: read_eeprom,
		(*param)[1] == 'i' || (*param)[1] == 'I'
		? ihex_output
		: srec_output))
	return 2;
      param++;
      break;
    case 'v':
    case 'V':
    case 'l':
    case 'L':
      if (param + 1 >= &argv[argc])
	goto Missing;
      if (!comm)
	goto Comm;
      if (!avr)
	goto Detect;
      else {
	pdest_t dest = 0;
	int (*done) (const struct comm*, const struct avr*) = 0;
	switch ((*param)[1]) {
	case 'v': dest = verify_flash; break;
	case 'V': dest = verify_eeprom; break;
	case 'l': dest = write_flash, done = flush_flash; break;
	case 'L': dest = write_eeprom; break;
	}
	if (load (param[1], dest, done))
	  return 2;
      }
      param++;
      break;
    case '?':
    case 'h':
      fputs ("cisp 1.0: compact in-system programmer\n"
	     "Options:\n"
	     "-c protocol device\t"
	     "Specify the communication protocol and interface\n"
	     "-C command\t"
	     "Send a 4-byte command to the device\n"
	     "-e\t"
	     "Erase device\n"
	     "-l file\t"
	     "Upload the file to flash memory\n"
	     "-v file\t"
	     "Verify the file against flash memory\n"
	     "-s file\t"
	     "Download the flash memory to a Motorola S record file\n"
	     "-i file\t"
	     "Download the flash memory to an Intel hex file\n"
	     "-L file\t"
	     "Upload the file to EEPROM\n"
	     "-V file\t"
	     "Verify the file against EEPROM\n"
	     "-S file\t"
	     "Download the EEPROM to a Motorola S record file\n"
	     "-I file\t"
	     "Download the EEPROM to an Intel hex file\n"
	     "-h\t"
	     "Get this help message\n", stderr);
      return 0;

    default:
    Unrecognized:
      fprintf (stderr, "cisp: unrecognized option `%s'\n", *param);
    Usage:
      fputs ("Type \"cisp -h\" to get help.\n", stderr);
      return 1;
    }
  }

  if (!comm)
    goto Usage;

  return 0;
}

==> cisp-1.0.2.tar.gz: cisp-1.0.2/cisp.c <==
/**
 * @file cisp.c
 * Compact In System Programmer
 * @author Marko Mkel (msmakela@nic.funet.fi)
 */

/* Copyright  2002 Marko Mkel.

   This file is part of CISP, a utility that allows in-system programming
   of electronic components.

   CISP is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   CISP is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   The GNU General Public License is often shipped with GNU software, and
   is generally kept in a file called COPYING or LICENSE.  If you do not
   have a copy of the license, write to the Free Software Foundation,
   59 Temple Place, Suite 330, Boston, MA 02111 USA. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <errno.h>

#include "comm.h"
#include "commsel.h"
#include "spi.h"
#include "ihex.h"
#include "srec.h"

/** Physical data source */
typedef int (*psrc_t) (const struct comm*, const struct avr*,
		       unsigned, unsigned char*);
/** Physical data destination */
typedef int (*pdest_t) (const struct comm*, const struct avr*,
			unsigned, unsigned char);
/** Stream data destination */
typedef int (*sdest_t) (FILE*, const struct comm*, const struct avr*,
			unsigned, psrc_t);

/** Signal handler
 * @param num	the signal number
 */
static void
sig (int num)
{
  terminate ();
  signal (num, SIG_DFL);
  raise (num);
}

/** the communication primitives */
static const struct comm* comm = 0;
/** the detected device */
static const struct avr* avr = 0;

/** Save a memory area
 * @param len	the length of the memory area
 * @param name	the file name ('-'=stdout)
 * @param src	the data source
 * @param dest	the data destination
 * @return	zero on success, nonzero otherwise
 */
static int
save (unsigned len,
      char* name,
      psrc_t src,
      sdest_t dest)
{
  int st;
  FILE* f;
  f = *name == '-' && !name[1] ? stdout : fopen (name, "w");
  if (!f) {
    fputs (name, stderr), perror (": fopen(writing)");
    return -1;
  }

  st = (*dest) (f, comm, avr, len, src);
  if (f != stdout)
    fclose (f);
  return st;
}

/** Load a memory area
 * @param name	the file name ('-'=stdin)
 * @param dest	the data destination
 * @param done	function to invoke at the end of data
 * @return	zero on success, nonzero otherwise
 */
static int
load (char* name,
      pdest_t dest,
      int (*done) (const struct comm*, const struct avr*))
{
  int c;
  FILE* f = *name == '-' && !name[1] ? stdin : fopen (name, "r");
  if (!f) {
    fputs (name, stderr), perror (": fopen(reading)");
    return -1;
  }
  c = getc (f);
  ungetc (c, f);
  switch (c) {
  default:
    fputs (name, stderr), fputs (": unknown format\n", stderr);
    c = EOF;
    break;
  case EOF:
    c = 0;
    break;
  case ':':
    c = ihex_input (f, comm, avr, dest, done);
    break;
  case 'S':
    c = srec_input (f, comm, avr, dest, done);
    break;
  }
  if (f != stdin)
    fclose (f);
  return c;
}

/** The main function
 * @param argc	argument count
 * @param argv	argument vector
 * @return	0 if successful
 */
int
main (int argc, char** argv)
{
  char** param;
  atexit (terminate);
  signal (SIGINT, sig);
  signal (SIGTERM, sig);

#ifndef NO_SIGNAL
  signal (SIGINT, sig);
  signal (SIGTERM, sig);
#endif /* NO_SIGNAL */

  /* process the command-line parameters */
  for (param = argv; ++param < &argv[argc]; ) {
    if (**param != '-' || !(*param)[1] || (*param)[2])
      goto Unrecognized;

    switch ((*param)[1]) {
    case 'c':
      if (param + 2 >= &argv[argc]) {
      Missing:
	fprintf (stderr, "%s: argument missing\n", *param);
	goto Usage;
      }
      if (comm)
	terminate ();
      if (!(comm = establish (param[1], param[2])))
	return 2;
      param += 2;
    reset:
      if (enable (comm)) {
	fputs ("cisp: failed to enable the programming mode\n", stderr);
	return 3;
      }
      if ((avr = detect (comm)))
	fputs (avr->name, stderr), fputs (" detected\n", stderr);
      break;
    case 'C':
      if (param + 1 >= &argv[argc])
	goto Missing;
      else if (!comm) {
      Comm:
	fprintf (stderr, "%s: no -c parameter given\n", *param);
	goto Usage;
      }
      else if (strlen (param[1]) != 8) {
      nothex:
	fprintf (stderr, "-C: expecting 8 hexadecimal characters, not %s\n",
		 param[1]);
	goto Usage;
      }
      else {
	unsigned char cmd[4], recv[4];
	unsigned i;
	for (i = 0; i < 8; i++) {
	  unsigned char digit = param[1][i];
	  if (digit >= '0' && digit <= '9')
	    digit -= '0';
	  else if (digit >= 'A' && digit <= 'F')
	    digit -= 'A' - 10;
	  else if (digit >= 'a' && digit <= 'f')
	    digit -= 'a' - 10;
	  else
	    goto nothex;
	  if (i & 1)
	    cmd[i >> 1] |= digit;
	  else
	    cmd[i >> 1] = digit << 4;
	}
	sendrecv (comm, cmd, recv, "-C");
	fprintf (stderr, "-C %02X%02X%02X%02X: got %02X%02X%02X%02X\n",
		 cmd[0], cmd[1], cmd[2], cmd[3],
		 recv[0], recv[1], recv[2], recv[3]);
      }
      param++;
      break;
    case 'e':
      if (!comm)
	goto Comm;
      if (erase (comm)) {
	fputs ("cisp: failed to erase the device\n", stderr);
	return 3;
      }
      goto reset;
      break;
    case 's': case 'i':
    case 'S': case 'I':
      if (param + 1 >= &argv[argc])
	goto Missing;
      if (!comm)
	goto Comm;
      if (!avr) {
      Detect:
	fprintf (stderr, "%s: no device has been detected\n", *param);
	goto Usage;
      }
      if (save ((*param)[1] == 's' || (*param)[1] == 'i'
		? avr->flash_size
		: avr->eeprom_size,
		param[1],
		(*param)[1] == 's' || (*param)[1] == 'i'
		? read_flash
		: read_eeprom,
		(*param)[1] == 'i' || (*param)[1] == 'I'
		? ihex_output
		: srec_output))
	return 2;
      param++;
      break;
    case 'v':
    case 'V':
    case 'l':
    case 'L':
      if (param + 1 >= &argv[argc])
	goto Missing;
      if (!comm)
	goto Comm;
      if (!avr)
	goto Detect;
      else {
	pdest_t dest = 0;
	int (*done) (const struct comm*, const struct avr*) = 0;
	switch ((*param)[1]) {
	case 'v': dest = verify_flash; break;
	case 'V': dest = verify_eeprom; break;
	case 'l': dest = write_flash, done = flush_flash; break;
	case 'L': dest = write_eeprom; break;
	}
	if (load (param[1], dest, done))
	  return 2;
      }
      param++;
      break;
    case '?':
    case 'h':
      fputs ("cisp 1.0.2: compact in-system programmer\n"
	     "Options:\n"
	     "-c protocol device\t"
	     "Specify the communication protocol and interface\n"
	     "-C command\t"
	     "Send a 4-byte command to the device\n"
	     "-e\t"
	     "Erase device\n"
	     "-l file\t"
	     "Upload the file to flash memory\n"
	     "-v file\t"
	     "Verify the file against flash memory\n"
	     "-s file\t"
	     "Download the flash memory to a Motorola S record file\n"
	     "-i file\t"
	     "Download the flash memory to an Intel hex file\n"
	     "-L file\t"
	     "Upload the file to EEPROM\n"
	     "-V file\t"
	     "Verify the file against EEPROM\n"
	     "-S file\t"
	     "Download the EEPROM to a Motorola S record file\n"
	     "-I file\t"
	     "Download the EEPROM to an Intel hex file\n"
	     "-h\t"
	     "Get this help message\n", stderr);
      return 0;

    default:
    Unrecognized:
      fprintf (stderr, "cisp: unrecognized option `%s'\n", *param);
    Usage:
      fputs ("Type \"cisp -h\" to get help.\n", stderr);
      return 1;
    }
  }

  if (!comm)
    goto Usage;

  return 0;
}
